<?php
// Database connection
$servername = "localhost";
$username = "packetlogger";
$password = "packetlogger";
$database = "packetlogger";

$conn = mysqli_connect($servername, $username, $password, $database);

// Check connection
if (!$conn) {
    die("Connection failed: " . mysqli_connect_error());
}


$deviceFilter = isset($_GET['device']) ? $_GET['device'] : null;
$gatewayFilter = isset($_GET['gateway']) ? $_GET['gateway'] : null;


$whereClause = "";
if ($deviceFilter) {
    $whereClause = "WHERE device_addr = '" . mysqli_real_escape_string($conn, $deviceFilter) . "'";
} elseif ($gatewayFilter) {
    $whereClause = "WHERE gateway_eui = '" . mysqli_real_escape_string($conn, $gatewayFilter) . "'";
}

// Total devices (distinct device_addr)
$devicesQuery = "SELECT COUNT(DISTINCT device_addr) AS total_devices FROM devicelog $whereClause";
$devicesResult = mysqli_query($conn, $devicesQuery);

if ($devicesResult) {
    $devicesRow = mysqli_fetch_assoc($devicesResult);
    $totalDevices = $devicesRow['total_devices'];
} else {
    die("Error in query: " . mysqli_error($conn));
}

// Total messages (sum of packet_counter)
$messagesQuery = "SELECT SUM(packet_counter) AS total_messages FROM devicelog $whereClause";
$messagesResult = mysqli_query($conn, $messagesQuery);

if ($messagesResult) {
    $messagesRow = mysqli_fetch_assoc($messagesResult);
    $totalMessages = $messagesRow['total_messages'];
} else {
    die("Error in query: " . mysqli_error($conn));
}

// First and last entries (datetime)
$firstEntryQuery = "SELECT MIN(datetime) AS first_entry FROM devicelog $whereClause";
$lastEntryQuery = "SELECT MAX(datetime) AS last_entry FROM devicelog $whereClause";

$firstEntryResult = mysqli_query($conn, $firstEntryQuery);
$lastEntryResult = mysqli_query($conn, $lastEntryQuery);

$firstEntry = ($firstEntryResult && mysqli_num_rows($firstEntryResult) > 0) 
    ? mysqli_fetch_assoc($firstEntryResult)['first_entry'] 
    : "N/A";
$lastEntry = ($lastEntryResult && mysqli_num_rows($lastEntryResult) > 0) 
    ? mysqli_fetch_assoc($lastEntryResult)['last_entry'] 
    : "N/A";

// Network statistics
$networkQuery = "
    SELECT 
        network, 
        COUNT(*) AS count, 
        (COUNT(*) / (SELECT COUNT(*) FROM devicelog $whereClause) * 100) AS percentage 
    FROM devicelog 
    $whereClause
    GROUP BY network 
    ORDER BY count DESC
";
$networkResult = mysqli_query($conn, $networkQuery);

if (!$networkResult) {
    die("Error in query: " . mysqli_error($conn));
}

// Device-specific data
if ($deviceFilter) {
    $deviceDetailsQuery = "
        SELECT datetime, device_addr, network, fcnt, packet_counter, dir, rssi, snr, gateway_eui, payload 
        FROM devicelog 
        WHERE device_addr = '" . mysqli_real_escape_string($conn, $deviceFilter) . "'
        ORDER BY datetime DESC
    ";
    $deviceDetailsResult = mysqli_query($conn, $deviceDetailsQuery);

    if (!$deviceDetailsResult) {
        die("Error in query: " . mysqli_error($conn));
    }
    $deviceDataExists = mysqli_num_rows($deviceDetailsResult) > 0;
}

// Last entries and top packet counters (only if ?device=XXX is NOT set)
if (!$deviceFilter) {
    $lastEntriesQuery = "
        SELECT datetime, device_addr, rssi, gateway_eui 
        FROM devicelog 
        $whereClause
        ORDER BY datetime DESC 
        LIMIT 10
    ";
    $lastEntriesResult = mysqli_query($conn, $lastEntriesQuery);

    if (!$lastEntriesResult) {
        die("Error in query: " . mysqli_error($conn));
    }

    $highestPacketQuery = "
        SELECT packet_counter, device_addr, rssi, gateway_eui 
        FROM devicelog 
        $whereClause
        ORDER BY packet_counter DESC 
        LIMIT 10
    ";
    $highestPacketResult = mysqli_query($conn, $highestPacketQuery);

    if (!$highestPacketResult) {
        die("Error in query: " . mysqli_error($conn));
    }
}
?>

<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>Network Statistics</title>
<style>
    body {
        font-family: Arial, sans-serif;
        margin: 0;
        padding: 5px;
        background-color: #f9f9f9;
        color: #333;
    }
    h1, h2 {
        text-align: center;
        color: #007BFF;
        margin: 20px;
    }
    p {
        text-align: center;
        margin: 10px 0;
    }
    .table-container {
        margin: 20px auto;
        padding: 10px;
        width: 90%;
        overflow-x: auto;
        border: 1px solid #ddd;
        border-radius: 8px;
        box-shadow: 0 4px 8px rgba(0, 0, 0, 0.1);
        background-color: #fff;
    }
    table {
        width: 100%;
        max-width: 80%;
        margin: 0 auto;
        border-collapse: collapse;
    }
    th, td {
        border: 1px solid #ddd;
        padding: 12px;
        text-align: center;
    }
    th {
        background-color: #007BFF;
        color: white;
    }
    tr:nth-child(even) {
        background-color: #f2f2f2;
    }
    tr:hover {
        background-color: #e9f5ff;
    }
    a {
        color: #007BFF;
        text-decoration: none;
    }
    a:hover {
        text-decoration: underline;
    }
    @media (max-width: 768px) {
        .table-container {
            width: calc(100% - 6px); /* Abstand von 3px auf beiden Seiten */
            margin: 20px auto;
            padding: 10px;
        }
        table {
            max-width: 100%;
        }
        th, td {
            padding: 8px;
        }
        h1, h2 {
            font-size: 1.5rem;
        }
    }
</style>
</head>
<body>
    <h1 style="text-align: center;"><a href="index.php">LoRaWAN Packet Recorder</a></h1>

    <?php if ($deviceFilter): ?>
        <!-- Device-specific data -->
        <h2 style="text-align: center;">Details for Device: <?php echo htmlspecialchars($deviceFilter); ?></h2>
        <?php if ($deviceDataExists): ?>
            <table>
                <thead>
                    <tr>
                        <th>Timestamp</th>
                        <th>Device Address</th>
                        <th>Network</th>
                        <th>Frame Counter</th>
                        <th>Packet Counter</th>
                        <th>Gateway EUI</th>
                        <th>RSSI</th>
                        <th>SNR</th>
                        <th>Payload</th>

                    </tr>
                </thead>
                <tbody>
                    <?php while ($row = mysqli_fetch_assoc($deviceDetailsResult)): ?>
                        <tr>
                            <td><?php echo htmlspecialchars($row['datetime']); ?></td>
                            <td><a href="?device=<?php echo htmlspecialchars($row['device_addr']); ?>"><?php echo htmlspecialchars($row['device_addr']); ?></a></td>
                            <td><?php echo htmlspecialchars($row['network']); ?></td>
                            <td><?php echo htmlspecialchars($row['fcnt']); ?></td>
                            <td><?php echo htmlspecialchars($row['packet_counter']); ?></td>
		     	    <td><a href="?gateway=<?php echo htmlspecialchars($row['gateway_eui']); ?>"><?php echo htmlspecialchars($row['gateway_eui']); ?></a></td>
                      	    <td><?php echo htmlspecialchars($row['rssi']); ?></td>
                            <td><?php echo htmlspecialchars($row['snr']); ?></td>
                            <td><?php echo htmlspecialchars($row['payload']); ?></td>
                         </tr>
                    <?php endwhile; ?>
                </tbody>
            </table>
        <?php else: ?>
            <p class="info">No data found for Device: <?php echo htmlspecialchars($deviceFilter); ?></p>
        <?php endif; ?>
    <?php else: ?>
        <!-- General or gateway-specific data -->
        <h2 style="text-align: center;">
            <?php echo $gatewayFilter ? "Details for Gateway: " . htmlspecialchars($gatewayFilter) : "General Statistics"; ?>
        </h2>
        <p style="text-align: center;">
            <b>Devices: <?php echo htmlspecialchars($totalDevices); ?></b><br>
            <b>Messages: <?php echo htmlspecialchars($totalMessages); ?></b><br>
            Timespan: From <?php echo htmlspecialchars($firstEntry) . ' to ' . htmlspecialchars($lastEntry); ?>
        </p>

        <!-- Network Statistics Table -->
        <h2 style="text-align: center;">Network Statistics</h2>
        <table>
            <thead>
                <tr>
                    <th>Network</th>
                    <th>Devices</th>
                    <th>%</th>
                </tr>
            </thead>
            <tbody>
                <?php while ($row = mysqli_fetch_assoc($networkResult)): ?>
                    <tr>
                        <td><?php echo htmlspecialchars($row['network']); ?></td>
                        <td><?php echo htmlspecialchars($row['count']); ?></td>
                        <td><?php echo htmlspecialchars(number_format($row['percentage'], 2)); ?>%</td>
                    </tr>
                <?php endwhile; ?>
            </tbody>
        </table>

        <!-- Last Entries Table -->
        <h2 style="text-align: center;">Last received Devices</h2>
        <table>
            <thead>
                <tr>
                    <th>Timestamp</th>
                    <th>Device Address</th>
                    <th>RSSI</th>
                    <th>Gateway EUI</th>
                </tr>
            </thead>
            <tbody>
                <?php while ($row = mysqli_fetch_assoc($lastEntriesResult)): ?>
                    <tr>
                        <td><?php echo htmlspecialchars($row['datetime']); ?></td>
                        <td><a href="?device=<?php echo htmlspecialchars($row['device_addr']); ?>"><?php echo htmlspecialchars($row['device_addr']); ?></a></td>
                        <td><?php echo htmlspecialchars($row['rssi']); ?></td>
                        <td><a href="?gateway=<?php echo htmlspecialchars($row['gateway_eui']); ?>"><?php echo htmlspecialchars($row['gateway_eui']); ?></a></td>
                    </tr>
                <?php endwhile; ?>
            </tbody>
        </table>

        <!-- Top Packet Counter Table -->
        <h2 style="text-align: center;">Devices by Packet Counter</h2>
        <table>
            <thead>
                <tr>
                    <th>Packet Counter</th>
                    <th>Device Address</th>
                    <th>RSSI</th>
                    <th>Gateway EUI</th>
                </tr>
            </thead>
            <tbody>
                <?php while ($row = mysqli_fetch_assoc($highestPacketResult)): ?>
                    <tr>
                        <td><?php echo htmlspecialchars($row['packet_counter']); ?></td>
                        <td><a href="?device=<?php echo htmlspecialchars($row['device_addr']); ?>"><?php echo htmlspecialchars($row['device_addr']); ?></a></td>
                        <td><?php echo htmlspecialchars($row['rssi']); ?></td>
                        <td><a href="?gateway=<?php echo htmlspecialchars($row['gateway_eui']); ?>"><?php echo htmlspecialchars($row['gateway_eui']); ?></a></td>
                    </tr>
                <?php endwhile; ?>
            </tbody>
        </table>
    <?php endif; ?>
</body>
</html>

<?php
// Free results and close connection
mysqli_free_result($devicesResult);
mysqli_free_result($messagesResult);
if (!$deviceFilter) {
    mysqli_free_result($networkResult);
    mysqli_free_result($lastEntriesResult);
    mysqli_free_result($highestPacketResult);
}
if ($deviceFilter && isset($deviceDetailsResult)) {
    mysqli_free_result($deviceDetailsResult);
}
if (isset($firstEntryResult)) {
    mysqli_free_result($firstEntryResult);
}
if (isset($lastEntryResult)) {
    mysqli_free_result($lastEntryResult);
}
mysqli_close($conn);
?>
