#include "src/network.h"
#include "src/functions.h"
#include "src/database_handler.h"
#include "src/configfile_handler.h"

#include <any>
#include <stdexcept>


    std::string configPath = "config.ini";

    // Database connection
    std::string db_server = "";
    std::string db_user = "";
    std::string db_password = "";
    std::string db_name = "";
    unsigned int db_port = 0;
    MYSQL* conn = nullptr;

void sendAckMessage(int udpSocket, const sockaddr_in& clientAddr, const char* receivedBuffer, const char ackMessage) {
    
    char ackPacket[4];
    ackPacket[0] = receivedBuffer[0];
    ackPacket[1] = receivedBuffer[1];
    ackPacket[2] = receivedBuffer[2];
    ackPacket[3] = ackMessage;

    // Send ACK packet
    int sentBytes = sendto(udpSocket, ackPacket, sizeof(ackPacket), 0, 
                           (sockaddr*)&clientAddr, sizeof(clientAddr));
    if (sentBytes != sizeof(ackPacket)) {
        logMessage(LogLevel::Error, "Error sending ACK to Gateway!");
    } else {
        logMessage(LogLevel::Verbose, "<- ACK sent to Gateway");
    }
}

std::vector<std::any> gatewayRxData(const char* buffer) {

std::vector<std::any> result;

            bool returnValid = false;

            std::string rx_gatewayeui = "";
            int rx_chan = 0;
            std::string rx_codr = "";
            std::string rx_data = "";
            std::string rx_datr = "";
            float rx_freq = 0.00;
            float rx_lsnr = 0.00;
            std::string rx_modu = "";
            int rx_rfch = 0;
            float rx_rssi = 0.00;
            int rx_size = 0;
            int rx_stat = 0;
            std::string rx_time = "";
            std::string rx_tmst = "";

            std::ostringstream oss;  
                for (int i = 4; i < 12; ++i) {
                oss << std::hex << std::setw(2) << std::setfill('0') << (int)(unsigned char)buffer[i];
                }

            rx_gatewayeui = oss.str();    

     if(getJsonValue(buffer + 12, "rxpk").length() > 0){
            rx_chan = stringToInt(getJsonValue(buffer + 12, "chan"));
            rx_codr = getJsonValue(buffer + 12, "codr");
            rx_data = getJsonValue(buffer + 12, "data");
            rx_datr = getJsonValue(buffer + 12, "datr");
            rx_freq = stringToFloat(getJsonValue(buffer + 12, "freq"));
            rx_lsnr = stringToFloat(getJsonValue(buffer + 12, "lsnr"));
            rx_modu = getJsonValue(buffer + 12, "modu");
            rx_rfch = stringToInt(getJsonValue(buffer + 12, "rfch"));
            rx_rssi = stringToFloat(getJsonValue(buffer + 12, "rssi"));
            rx_size = stringToInt(getJsonValue(buffer + 12, "size"));
            rx_stat = stringToInt(getJsonValue(buffer + 12, "stat"));
            rx_time = getJsonValue(buffer + 12, "time");
            rx_tmst = getJsonValue(buffer + 12, "tmst");
            returnValid = true;
     }

    result.push_back(returnValid);
    result.push_back(rx_gatewayeui);
    result.push_back(rx_chan);
    result.push_back(rx_codr);
    result.push_back(rx_data);
    result.push_back(rx_datr);
    result.push_back(rx_freq);
    result.push_back(rx_lsnr);
    result.push_back(rx_modu);
    result.push_back(rx_rfch);
    result.push_back(rx_rssi);
    result.push_back(rx_size);
    result.push_back(rx_stat);
    result.push_back(rx_time);
    result.push_back(rx_tmst);
    return result;

}

std::vector<std::any> lorawanUplinkMessage(const char* buffer) {
    std::vector<std::any> result;

    bool returnValid = false;
    std::string up_devaddr = "";
    int up_fport = 0;
    int up_fcnt = 0;
    std::string up_enc_hex_payload = "";
    std::string up_enc_hex_mic = "";

    try {
        if (getJsonValue(buffer + 12, "rxpk").length() > 0) {
            std::ostringstream oss;

            std::string str_up_phypayload = getJsonValue(buffer + 12, "data");
            if (str_up_phypayload.empty()) {
                logMessage(LogLevel::Error, "Uplink Message: Base64 data string is empty.");
            }

            std::vector<uint8_t> hex_up_phypayload = base64ToHexArray(str_up_phypayload);
            size_t hex_up_phypayload_size = hex_up_phypayload.size();

            if (hex_up_phypayload_size < 9) {
                logMessage(LogLevel::Error, "Uplink Message: Payload size is too small.");
            }

            uint8_t fhdr = hex_up_phypayload[5]; // Byte 5 = FHDR byte

           
            bool adr = fhdr & 0x80;          // Bit 7 is ADR
            bool ack = fhdr & 0x20;          // Bit 5 is ACK
            bool fPending = fhdr & 0x10;     // Bit 4 is FPending

            uint8_t fol = fhdr & 0x0F;  // Bits 3..0 is FOptsLen
            int fOptsLen = static_cast<int>(fol);

            if (8 + fOptsLen >= hex_up_phypayload_size) {
                logMessage(LogLevel::Error, "Uplink Message: FOptsLen exceeds payload size.");
            }

            int fPortIndex = 8 + fOptsLen;
            up_fcnt = static_cast<int>(((hex_up_phypayload[7]) << 8) | hex_up_phypayload[6]);
            up_fport = hex_up_phypayload[fPortIndex];

            // Device address in hex
            oss.str("");
            for (int i = 4; i >= 1; --i) {
                oss << std::hex << std::setw(2) << std::setfill('0')
                    << static_cast<int>(hex_up_phypayload[i]);
            }
            up_devaddr = oss.str();

            // Encrypted payload in hex
            oss.str("");
            for (int i = 9 + fOptsLen; i < hex_up_phypayload_size - 4; ++i) {
                if (i < 0 || i >= hex_up_phypayload_size) {
                    logMessage(LogLevel::Error, "Uplink Message: Payload index out of bounds.");
                    break;
                }
                oss << std::hex << std::setw(2) << std::setfill('0')
                    << static_cast<int>(hex_up_phypayload[i]);
            }
            up_enc_hex_payload = oss.str();

            // MIC in hex
            oss.str("");
            for (int i = hex_up_phypayload_size - 4; i < hex_up_phypayload_size; ++i) {
                oss << std::hex << std::setw(2) << std::setfill('0')
                    << static_cast<int>(hex_up_phypayload[i]);
            }
            up_enc_hex_mic = oss.str();

            logMessage(LogLevel::Verbose, "Uplink Message: Payload successfully processed.");
            returnValid = true;
        } else {
            logMessage(LogLevel::Normal, "UDP Message: No rxpk field found in buffer.");
        }
    } catch (const std::exception& ex) {
        logMessage(LogLevel::Error, std::string("Uplink Message: ") + ex.what());
    } catch (...) {
        logMessage(LogLevel::Error, "Uplink Message: Unknown error in lorawanUplinkMessage.");
    }

    result.push_back(returnValid);
    result.push_back(up_devaddr);
    result.push_back(up_fport);
    result.push_back(up_fcnt);
    result.push_back(up_enc_hex_payload);
    result.push_back(up_enc_hex_mic);
    return result;
}

int lorawanMessageType(const char* buffer) {
    
    int messageType = 0;

    if (getJsonValue(buffer + 12, "rxpk").length() > 0) {
        std::string str_up_phypayload = getJsonValue(buffer + 12, "data");
        std::vector<uint8_t> hex_up_phypayload = base64ToHexArray(str_up_phypayload);

        //Message type from Byte 0
        uint8_t mhdr = hex_up_phypayload[0];
        uint8_t mtype = (mhdr >> 5) & 0x07;

        // 1=Join, 2=Join Accept, 3=Unconfirmed Up, 4=Unconfirmed Down, 5=Confirmed Up, 6=Confirmed Down, 7=Rejon, 8=Proprietary

        switch (mtype) {
        case 0b000:
            messageType = 1;
            break;
        case 0b001:
            messageType = 2;
            break;
        case 0b010:
            messageType = 3;
            break;
        case 0b011:
            messageType = 4;
            break;
        case 0b100:
            messageType = 5;
            break;
        case 0b101:
            messageType = 6;
            break;
        case 0b110:
            messageType = 7;
            break;
        case 0b111:
            messageType = 8;
            break;
        }
    }
        return messageType;
}



void processReceivedMessage(const char* buffer, int length) {
   try{
    if (length > 12) {
      int mType = lorawanMessageType(buffer);

    //Handle uplink messages
    if(mType == 3 || mType == 5){ 

    auto extracted_gateway_rx = gatewayRxData(buffer);
    auto extracted_uplink_msg = lorawanUplinkMessage(buffer);

    if(std::any_cast<bool>(extracted_gateway_rx[0]) && std::any_cast<bool>(extracted_uplink_msg[0])){

    std::string device_addr = std::any_cast<std::string>(extracted_uplink_msg[1]);  
    std::string up_enc_payload = std::any_cast<std::string>(extracted_uplink_msg[4]);
    std::string network = getNetId(device_addr);
    std::string frame_port = std::to_string(std::any_cast<int>(extracted_uplink_msg[2]));
    std::string frame_counter = std::to_string(std::any_cast<int>(extracted_uplink_msg[3]));
    std::string dir = "up";
    float rssi = std::any_cast<float>(extracted_gateway_rx[10]);
    float snr = std::any_cast<float>(extracted_gateway_rx[7]);
    std::string gateway_eui = std::any_cast<std::string>(extracted_gateway_rx[1]);  
    std::string datetime = getLocalTimestamp();

    std::string fullLogMessage = datetime + " -> Uplink Message from Device '" + device_addr + "' via Gateway '" + gateway_eui + "' received.";
    logMessage(LogLevel::Normal, fullLogMessage);

    // Insert packet in database
    if(check_db_connection(db_server, db_user, db_password, db_name, db_port, conn)){
        if (!db_insert_or_update_packet(conn, datetime, device_addr, network, frame_counter, dir, rssi, snr, gateway_eui, up_enc_payload)) {
        logMessage(LogLevel::Error, "Failed to insert Packet!");
        }
    }
      }else{
        logMessage(LogLevel::Normal, "Uplink Message processing error, Message skipped!");
      }
    }else if(mType == 4 || mType == 6){
        //logMessage(LogLevel::Normal, "Downlink Message received... (RFU)");
    }
    }
   } catch (const std::exception& e) {
    std::string fullLogMessage = "Error in handling received Message: " + std::string(e.what());
    logMessage(LogLevel::Error, fullLogMessage);
    }
}

int main() {
    
    int udpSocket;
    sockaddr_in serverAddr;
    

    // Check config.ini exists
    if (!ConfigFileHandler::fileExists(configPath)) {
        std::cerr << "Config file does not exist: " << configPath << std::endl;
        std::cin.get();
        return 1;
    }

    // Load config.ini
    if (!ConfigFileHandler::loadConfig(configPath)) {
        std::cerr << "Failed to load config.ini file: " << configPath << std::endl;
        std::cin.get();
        return 1;
    }


    // Read parameters from config.ini
    db_server = ConfigFileHandler::getParam("Database", "host");
    db_port = ConfigFileHandler::getIntParam("Database", "port");
    db_user = ConfigFileHandler::getParam("Database", "user");
    db_password = ConfigFileHandler::getParam("Database", "password");
    db_name = ConfigFileHandler::getParam("Database", "dbname");
    int logLevelValue = ConfigFileHandler::getIntParam("Application", "loglevel");

      try {
        initializeLogger(logLevelValue);
    } catch (const std::exception& e) {
        std::cerr << "Initialization error: " << e.what() << std::endl;
        std::cin.get();
        return 1;
    }

    logMessage(LogLevel::Silent, "Packet Recorder V 1.0");

 
    if (!check_db_connection(db_server, db_user, db_password, db_name, db_port, conn)) {
        logMessage(LogLevel::Error, "Failed to connect to the database!");
        std::cout << "Press Enter to exit..." << std::endl;
        std::cin.get();
        return 1;
    }

    if (!check_table_exists(conn, db_name)) {
        logMessage(LogLevel::Error, "Failed to check if the table exists!");
        mysql_close(conn);
        std::cout << "Press Enter to exit..." << std::endl;
        std::cin.get();
        return 1;
    }

   
    if (!initializeNetwork()) {
        logMessage(LogLevel::Error, "Error: Network access not available!");
        std::cin.get();
        return 1;
    }

    logMessage(LogLevel::Normal, "Network access successful. Retrieving IPs:");
    printIPAddresses();


    if (initializeUDPListener(udpSocket, serverAddr) < 0) {
        logMessage(LogLevel::Error, "Failed to initialize UDP Listener!");
        return 1;
    }

    logMessage(LogLevel::Normal, "UDP Listener initialized. Waiting for packets...");

    char buffer[1024];
    sockaddr_in clientAddr;
    socklen_t clientAddrLen = sizeof(clientAddr);

    // UDP Receiver Loop
    while (true) {
        int recvLen = recvfrom(udpSocket, buffer, sizeof(buffer), 0, (sockaddr*)&clientAddr, &clientAddrLen);
        if (recvLen < 0) {
            logMessage(LogLevel::Normal, "Error receiving packet!");
            continue;
        }

        processReceivedMessage(buffer, recvLen);
        logMessage(LogLevel::Verbose, "-> Received Data from Gateway");

        if (buffer[3] == 0x00) {
            sendAckMessage(udpSocket, clientAddr, buffer, 0x01);
        } else if (buffer[3] == 0x02) {
            sendAckMessage(udpSocket, clientAddr, buffer, 0x04);
        }
    }

    closeUDPListener(udpSocket);
    return 0;
}